import tensorflow as tf
import numpy as np
from PIL import Image
import os

WEIGHTS_INIT_STDEV = .1

CURRENT_PATH = os.path.abspath(__file__)

def net(image):
    conv1 = _conv_layer(image, 32, 9, 1)
    conv2 = _conv_layer(conv1, 64, 3, 2)
    conv3 = _conv_layer(conv2, 128, 3, 2)
    resid1 = _residual_block(conv3, 3)
    resid2 = _residual_block(resid1, 3)
    resid3 = _residual_block(resid2, 3)
    resid4 = _residual_block(resid3, 3)
    resid5 = _residual_block(resid4, 3)
    conv_t1 = _conv_tranpose_layer(resid5, 64, 3, 2)
    conv_t2 = _conv_tranpose_layer(conv_t1, 32, 3, 2)
    conv_t3 = _conv_layer(conv_t2, 3, 9, 1, relu=False)
    preds = tf.nn.tanh(conv_t3) * 150 + 255./2
    return preds


def _conv_layer(net, num_filters, filter_size, strides, relu=True):
    weights_init = _conv_init_vars(net, num_filters, filter_size)
    strides_shape = [1, strides, strides, 1]
    net = tf.nn.conv2d(input=net, filter=weights_init, strides=strides_shape, padding='SAME')
    net = _instance_norm(net)
    if relu:
        net = tf.nn.relu(net)
    return net


def _conv_tranpose_layer(net, num_filters, filter_size, strides):
    weights_init = _conv_init_vars(net, num_filters, filter_size, transpose=True)

    batch_size, rows, cols, in_channels = [i for i in net.get_shape()]
    new_rows, new_cols = int(rows * strides), int(cols * strides)
    # new_shape = #tf.pack([tf.shape(net)[0], new_rows, new_cols, num_filters])

    new_shape = [batch_size, new_rows, new_cols, num_filters]
    tf_shape = tf.stack(new_shape)
    strides_shape = [1,strides,strides,1]

    net = tf.nn.conv2d_transpose(net, weights_init, tf_shape, strides_shape, padding='SAME')
    net = _instance_norm(net)
    return tf.nn.relu(net)


def _residual_block(net, filter_size=3):
    tmp = _conv_layer(net, 128, filter_size, 1)
    return net + _conv_layer(tmp, 128, filter_size, 1, relu=False)


def _instance_norm(net):
    batch, rows, cols, channels = [i for i in net.get_shape()]
    var_shape = [channels]
    mu, sigma_sq = tf.nn.moments(x=net, axes=[1, 2], keep_dims=True)
    shift = tf.Variable(tf.zeros(var_shape))
    scale = tf.Variable(tf.ones(var_shape))
    epsilon = 1e-3
    normalized = (net-mu)/(sigma_sq + epsilon)**(.5)
    return scale * normalized + shift


def _conv_init_vars(net, out_channels, filter_size, transpose=False):
    _, rows, cols, in_channels = [i for i in net.get_shape()]
    if not transpose:
        weights_shape = [int(filter_size), int(filter_size), int(in_channels), int(out_channels)]
    else:
        weights_shape = [int(filter_size), int(filter_size), int(out_channels), int(in_channels)]

    weights_init = tf.Variable(tf.random.truncated_normal(shape=weights_shape, stddev=WEIGHTS_INIT_STDEV, seed=1),
                               dtype=tf.dtypes.float32)

    return weights_init


def ffwd(data_in, checkpoint_dir, device_t, batch_size):

    img_shape = data_in.shape
    g = tf.Graph()
    batch_size = batch_size
    soft_config = tf.compat.v1.ConfigProto(allow_soft_placement=True)
    soft_config.gpu_options.allow_growth = True

    with g.as_default(), g.device(device_t), tf.compat.v1.Session(config=soft_config) as sess:
        batch_shape = (batch_size,) + img_shape
        img_placeholder = tf.compat.v1.placeholder(tf.float32, shape=batch_shape, name='img_placeholder')
        preds = net(img_placeholder)
        saver = tf.compat.v1.train.Saver()
        if os.path.isdir(checkpoint_dir):
            ckpt = tf.train.get_checkpoint_state(checkpoint_dir)
            if ckpt and ckpt.model_checkpoint_path:
                saver.restore(sess, ckpt.model_checkpoint_path)
            else:
                raise Exception("No checkpoint found...")
        else:
            saver.restore(sess, checkpoint_dir)

        X = np.zeros(batch_shape, dtype=np.float32)
        img = data_in
        X[0] = img
        _preds = sess.run(preds, feed_dict={img_placeholder: X})
        img = np.clip(_preds[0], 0, 255).astype(np.uint8)
        img = Image.fromarray(img.astype('uint8')).convert('RGB')

    return img


def process(img, model="la_muse"):

    model = os.path.join(os.path.dirname(os.path.abspath(__file__)), "models", f"{model}.ckpt")
    img = np.array(img)
    result_img = ffwd(data_in=img, checkpoint_dir=model, device_t=None, batch_size=1)
    return result_img



from flask_socketio import SocketIO, emit
from ..http_server import app

socketio_app = SocketIO(app, logger=True, engineio_logger=True)
users = []
messages = []


@socketio_app.on('send message')
def send_message(message):
    print("receive a chat message")
    print(message)
    messages.append(message)
    emit('server response', {'status': 1})


@socketio_app.on('get message')
def get_message(message):
    print(f"client[{message['name']} get message]")
    print("messages:")
    print(messages)
    for i in messages:
        if i["receiver"] == message['name']:
            messages.remove(i)
            print(message)
            emit('message', i)


@socketio_app.on('user login')
def login(message):
    print(f"{message['name']} login")
    emit('server response', {'status': 1})
    emit('new user', {"name": message['name']}, broadcast=True)
    users.append(message['name'])


@socketio_app.on('connect')
def test_connect():
    print("connect")
    emit('server response', {'data': 'Connected'})


@socketio_app.on('disconnect')
def test_disconnect():
    print('Client disconnected')



#!/usr/bin/env python
# coding: utf-8

import json
import threading
from socket import *

SERVER_IP = '127.0.0.1'  # or 'localhost'
SERVER_PORT = 21567

HOST_IP = 'localhost'
USER_PORT = 9992
BUFSIZ = 1024
ADDR1 = (SERVER_IP, SERVER_PORT)
ADDR2 = (HOST_IP, USER_PORT)

def send_msg(host, port):

    tcpCliSock = socket(AF_INET, SOCK_STREAM)
    tcpCliSock.connect((host, port))
    data2 = input('>')
    data1 = [1, 'liu', 'meng']
    data1.append(data2)
    data1 = json.dumps(data1)
    tcpCliSock.send(data1.encode())
    tcpCliSock.close()
    return

def get_user_list(host, port):

    tcpCliSock = socket(AF_INET, SOCK_STREAM)
    print(f"connect to server: {host, port}")
    tcpCliSock.connect((host, port))
    data1 = [0]
    data2 = input('>')
    data1.append(data2)
    data3 = '0.0.0.0'
    data1.append(data3)
    data4 = input('>')
    data1.append(data4)
    data1 = json.dumps(data1)
    tcpCliSock.send(data1.encode())
    data1 = tcpCliSock.recv(BUFSIZ)
    data1 = json.loads(data1)
    print(data1)
    tcpCliSock.close()
    return data1


# 这里是开启一个监听端口
def User_Receive():
    global ADDR2
    tcpSerSock = socket(AF_INET, SOCK_STREAM)
    print(ADDR2)
    tcpSerSock.bind(ADDR2)
    tcpSerSock.listen(5)

    while True:
        print('waiting for connection...')
        tcpCliSock, addr = tcpSerSock.accept()
        print('...connecting from:', addr)

        while True:
            data = tcpCliSock.recv(BUFSIZ)
            if not data:
                break
            data = json.loads(data)
            print(data[0] + ':' + data[1])
        tcpCliSock.close()
    tcpSerSock.close()


def run_client(port=SERVER_PORT, host=SERVER_IP):

    thread = threading.Thread(target=User_Receive)
    thread.start()
    while True:
        s = input('>-------------------\n'
                  '1. get_user_list\n'
                  '2. send_message\n')
        if s == '1':
            a = get_user_list(host, port)
        else:
            send_msg(host, port)




from threading import Timer, Event, Thread

import socketio

# standard Python
sio = socketio.Client(logger=True, engineio_logger=True)

@sio.event
def message(data):
    print('I received a message!')

@sio.on('message')
def on_message(data):

    print('I received a message!')
    print(data)


@sio.on('new user')
def on_message(data):
    print(f"new user: {data['name']}")
    print('I received a message!')


@sio.on('server response')
def on_message(data):
    print(data)
    print('I received a message!')


@sio.event
def connect():
    print("I'm connected!")

@sio.event
def connect_error():
    print("The connection failed!")

@sio.event
def disconnect():
    print("I'm disconnected!")


class HeartbeatThread(Thread):
    """心跳"""

    def __init__(self, event, sio, my_name):
        super(HeartbeatThread, self).__init__()
        self.event = event
        self.sio = sio
        self.my_name = my_name

    def run(self):
        while 1:
            # 发送ping包
            self.sio.emit('get message', {'name': self.my_name})
            self.event.wait(timeout=2)

def on_emit(sio, my_name):
    # 创建心跳线程
    event = Event()
    heartbeat = HeartbeatThread(event, sio, my_name)
    heartbeat.start()

    while 1:
        inp = input("input: receiver,content")
        receiver, content = inp.split(",")
        # 发送信息
        # 4: engine.io message
        # 2: socket.io event
        # chat message event message
        sio.emit('send message', {'sender': my_name, 'receiver': receiver, "content": content})
        sio.sleep(.2)


def socketio_client(my_name, port):

    sio.connect(f'http://localhost:{port}')
    sio.emit('user login', {"name": my_name})

    t = Timer(3, on_emit, args=(sio, my_name))
    t.start()



from PIL import Image
from .yuchunlib import process
from flask import Flask, request
from flask import send_file
import os

CURRENT_PATH = os.path.abspath(__file__)

app = Flask(__name__)

@app.route('/photo/process', methods=['POST'])
def get_photos():
    file = request.files.get("file")
    model = request.args.get("model", "la_muse")

    print("{} - {}".format(file, file.filename))
    if file is None:
        raise Exception("文件数据丢失, 请重新上传")

    try:
        image = Image.open(file)

        tmp_image_path = os.path.join(os.path.dirname(os.path.abspath(__file__)), "tmp2.jpg")
        image.save(tmp_image_path)

        width, height = image.size
        print(f"图片大小为:{width} * {height}")
        image = resize_images(image, 160000)

        image = process(image, model=model)
        print(os.path.join(os.path.dirname(CURRENT_PATH), "tmp.jpg"))
        tmp_image_path = os.path.join(os.path.dirname(os.path.abspath(__file__)), "tmp.jpg")
        image.save(tmp_image_path)
        response = send_file(tmp_image_path,
                             mimetype='image/jpeg')
    except Exception as e:
        print("处理失败")
        raise e

    return response


def resize_images(image,  threshold):
    import math
    width, height = image.size
    if width * height >= threshold:
        s = math.sqrt(width * height / threshold)

        new_height = int(height / s)
        new_width = int(width / s)
        image = image.resize((new_width, new_height))
        print(f"需要压缩，new size: {new_width} * {new_height}={new_width * new_height}")
    return image


#!/usr/bin/env python
# coding: utf-8


from socket import *
import json

# 维护一个字典 key 是uid   value 是[ip,port] - server端口
user_dict = {'liu': ['127.0.0.1', 9991],
             'meng': ['127.0.0.1', 9992],
             'yang': ['127.0.0.1', 9993],
             'wo': ['127.0.0.1', 9994],
             'bu': ['127.0.0.1', 9995],
             'xiang': ['127.0.0.1', 9996],
             'nu': ['127.0.0.1', 9997],
             'li': ['127.0.0.1', 9998],
             'le': ['127.0.0.1', 9999],
             }

HOST = ''
PORT = 21567
BUFSIZ = 1024


def run_server(host=HOST, port=PORT):
    tcpSerSock = socket(AF_INET, SOCK_STREAM)
    print(f"bind {host, port}")
    tcpSerSock.bind((host, port))
    tcpSerSock.listen(5)

    while True:
        print('waiting for connection...')
        tcpCliSock, addr = tcpSerSock.accept()
        print('...connnecting from:', addr)

        while True:
            data = tcpCliSock.recv(BUFSIZ)
            #    [ flag,user1_name,user2_name,data]
            #    1是发送消息 0是获取用户列表
            if not data:
                break
            data = json.loads(data)
            print(data)
            if data[0] == 1:
                msg_sender = data[1]
                msg_receiver = data[2]
                msg_data = data[3]
                #         msg格式 [sender_name , data]
                msg = []
                msg.append(msg_sender)
                msg.append(msg_data)
                msg = json.dumps(msg)
                receiver_addr = user_dict[data[2]]
                print(receiver_addr)
                addr = (receiver_addr[0], receiver_addr[1])
                tcp_CliSock = socket(AF_INET, SOCK_STREAM)
                tcp_CliSock.connect(addr)
                tcp_CliSock.send(msg.encode())
                tcp_CliSock.close()

            else:
                user_name = data[1]
                if user_name in user_dict:
                    user_dict[user_name] = [data[2], data[3]]
                    print(user_dict[user_name])
                    data = json.dumps(list(user_dict.keys()))
                    tcpCliSock.send(data.encode())
                else:
                    msg = "user_name error"
                    data = json.dumps(msg)
                    tcpCliSock.send(data.encode())
        tcpCliSock.close()
    tcpSerSock.close()



# -*- coding: utf-8 -*-
import glob
from setuptools import setup, find_packages

try:  # for pip >= 10
    from pip._internal.req import parse_requirements
except ImportError:  # for pip <= 9.0.3
    from pip.req import parse_requirements
#
# with open('README.md') as f:
#     readme = f.read()

requirements = [
    "pymongo",
    "logbook",
    "flask",
    "click",
    "prettytable",
    "tensorflow==1.13.1",
    "imageio==2.9.0",
    "numpy==1.16.2",
    "Pillow==7.2.0",
    "flask-socketio",
    "python-socketio",
    "Pillow",
    "click"
]
setup(
    version="1.0.3",
    name="PixelInstagram",
    description="a simple app",
    # long_description=readme,
    keywords="PixelInstagram",
    include_package_data=True,
    packages=find_packages(include=["PixelInstagram", "PixelInstagram.*"]),
    install_requires=requirements,
    python_requires=">=3.6",
    classifiers=[
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Intended Audience :: Developers",
        "Operating System :: OS Independent",
        "Environment :: Console",
    ],
    zip_safe=False,
    entry_points={
        "console_scripts": [
            "PI = PixelInstagram.main:cli",
        ]
    },
    # long_description_content_type='text/markdown',
    author="wjh1119",
    author_email="20091755G@connect.polyu.hk",
    url='https://gitee.com/wjh1119'
)



from .yuchunlib import process
from PIL import Image
from .http_server import app
from .socketio_core.socketio_server import socketio_app
from .socketio_core.client import socketio_client
from .demo_client import run_client
from .socket_server import run_server

import click

miner_name = None


@click.group()
def cli():
    pass


@cli.command(name="debug")
def debug():
    img = process_img(Image.open('./chicago.jpg'))
    img.show()


@cli.command(name="flask")
@click.option("-p", "--port")
def flask(port=10155):
    app.run(port=port)


@cli.command(name="demo_client")
@click.option("-h", "--host")
@click.option("-p", "--port")
def socket_client(host="local_host", port=10155):
    run_client(host=host, port=int(port))


@cli.command(name="server")
@click.option("-p", "--port")
def socket_server(port=10155):
    run_server(host="localhost", port=int(port))


@cli.command(name="socketio_server")
@click.option("-p", "--port")
def socketio(port=10155):
    socketio_app.run(app, port=int(port))


@cli.command(name="socketio_client")
@click.option("-p", "--port")
@click.option("-n", "--name")
def client(name, port=10155):
    socketio_client(name, port)


def process_img(img):
    return process(img)
